/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.communication
 * File:    SOAPConnector.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.communication;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.List;

import org.apache.axis2.AxisFault;
import org.trustedcomputinggroup._2010.ifmap._2.DumpRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.DumpResponseType;
import org.trustedcomputinggroup._2010.ifmap._2.NewSessionRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.ObjectFactory;
import org.trustedcomputinggroup._2010.ifmap._2.PollRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.PollResultType;
import org.trustedcomputinggroup._2010.ifmap._2.ResponseType;
import org.trustedcomputinggroup._2010.ifmap._2.SessionRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.SubscribeRequestType;

import de.fhhannover.inform.ifmap.transform.MessageTransformer;

public class SOAPConnector {
	private IFMAPConnection ssrc = null;
	private IFMAPConnection arc = null;
	private String mIfmapPublisherID = null;
	private String mSessionID = null;
	private MessageTransformer transformer;
	private ObjectFactory mBaseFactory;
	private Calendar cal;
	private SimpleDateFormat sdf;
	
	public SOAPConnector(MessageTransformer t){
		transformer = t;
		mBaseFactory = new ObjectFactory();		
		cal = Calendar.getInstance();
		sdf = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss");
	}
	
	public String getPublisherID(){
		return mIfmapPublisherID;
	}
	
	public String getSessionID(){
		return mSessionID;
	}
	
	public synchronized void disconnect(){
		if(this.ssrc != null && this.arc != null){
			try {
				this.ssrc.cleanup();
				this.arc.cleanup();
				ssrc = null;
				arc = null;
			} catch (AxisFault e) {
				e.printStackTrace();
			}
		}		
	}
	
	public synchronized boolean connect()
	throws RemoteException{	
		if(this.ssrc == null){
			try {				
				ssrc = Connection.getInstance().newConnection();					
			} catch (AxisFault e) {					
				e.printStackTrace();
				transformer.notifyStatusChangedReceiver("Fehler beim newSession!");
				return false;
			} catch (FileNotFoundException e) {
				e.printStackTrace();
				return false;
			} catch (IOException e) {
				e.printStackTrace();
				return false;
			}			
		}
		else if(mIfmapPublisherID != null && mSessionID != null){
			return true;
		}
		try {
			transformer.notifyStatusChangedReceiver("Trying newSession()...");
			return newSession();
		} catch (RemoteException e) {
			transformer.notifyStatusChangedReceiver("Fehler beim newSession!");
			//e.printStackTrace();
			ssrc = null;
			mSessionID = null;			
			throw e;
		}
	}
	
	private synchronized boolean newSession() throws RemoteException {
		ResponseType response = null;
		NewSessionRequestType newSessioRequestType = mBaseFactory
				.createNewSessionRequestType();			
		response = ssrc.newSession(newSessioRequestType);				
		
		if (response.getErrorResult() == null) {
			/* System.out.println("Publisher-ID: "
					+ response.getNewSessionResult().getIfmapPublisherId());
			System.out.println("Session-ID: "
					+ response.getNewSessionResult().getSessionId()); */
			this.mSessionID = response.getNewSessionResult().getSessionId();
			this.mIfmapPublisherID = response.getNewSessionResult()
					.getIfmapPublisherId();
			transformer.notifyStatusChangedReceiver("newSession() OK!");
			return true;
		} else {			
			mSessionID = null;
			mIfmapPublisherID = null;
			transformer.notifyStatusChangedReceiver("newSession() Error!");			
			return false;
		}
	}
	
	public synchronized boolean subscribeUpdate(SubscribeRequestType srt) throws RemoteException{		
		if(!connect()){
			return false;
		}		
		ResponseType response = null;		
		srt.setSessionId(mSessionID);		
		response = this.ssrc.subscribe(srt);

		if (response.getErrorResult() == null) {			
			return true;
		}
		else return false;
	}

	/**
	 * This Method ends the Session with the IF-MAP Server 
	 * 
	 * @return true - if endSession was successful, false - otherwise 
	 * @throws RemoteException 
	 */
    private synchronized boolean endSession() {	
    	ResponseType response = null;    	
        SessionRequestType sessionRequestType = mBaseFactory.createSessionRequestType();
        sessionRequestType.setSessionId(mSessionID);	
        try{
			response = ssrc.endSession(sessionRequestType);
		} catch(RemoteException e){
			e.printStackTrace();
			ssrc = null;
			mSessionID = null;
			return false;
		}		
		mSessionID = null;
		if(response.getErrorResult() == null){						
			return true;
		} else {			
			return false;
		}
    }
    
    public PollResultType sendPoll()
    throws RemoteException{
		ResponseType response = null;
		PollRequestType poll = new PollRequestType();
		poll.setSessionId(mSessionID);	
		try {
			if(arc == null){
				arc = Connection.getInstance().newConnection();			
			}			
			response = arc.poll(poll);			
			if(response != null){
				if(response.getErrorResult() == null){
					return response.getPollResult();
				}
			}
		}
		 catch (AxisFault e1) {
			arc = null;
			e1.printStackTrace();
			throw new RemoteException();			
		} catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			arc = null;
			e1.printStackTrace();
		} catch (IOException e1) {
			arc = null;
			// TODO Auto-generated catch block
			e1.printStackTrace();		
		}		
		return null;
	}
    
    public List<Object> getDumpResult(String filter) 
    throws RemoteException{
    	DumpResponseType drt = dump(filter);
    	
    	if(drt == null) return null;
    	    	
    	List<Object> list = drt.getAccessRequestOrIdentityOrIpAddress();
    	if(list.size() == 0){			
			return null;
		}
    	else return list;
    }
    
    public long getLastUpdate() 
    throws RemoteException{
    	DumpResponseType drt = dump("-");    	    	
    	if(drt == null){			
			return 0;
		}		 
		 cal.setTimeInMillis(Long.parseLong(drt.getLastUpdate()));
		 transformer.notifyStatusChangedReceiver("Last update: "+sdf.format(cal.getTime()));
		 return Long.parseLong(drt.getLastUpdate());
	}
    
    private synchronized DumpResponseType dump(String filter) 
    throws RemoteException{    	
    	if(!connect()){
    		transformer.notifyStatusChangedReceiver("Connection error");
    		return null;    		
    	}
    	
		// send dump()-Request	
		ResponseType response = null;
		DumpRequestType dr = mBaseFactory.createDumpRequestType();
		dr.setSessionId(mSessionID);
		dr.setIdentifier(filter);
		try {
			response = ssrc.dump(dr);
		} catch (RemoteException e) {
			e.printStackTrace();
			ssrc = null;
			mSessionID = null;
			return null;
		}
		DumpResponseType resp = response.getDumpResult();		
		return resp;				
    }
}
